# coding=utf-8

import re
import urllib
import base64
import json
import zlib
import hashlib
import hmac
import sys
from copy import copy
import time
from datetime import datetime

from rest_client import RestClient
from websocket_client import WebsocketClient
from gateway import BaseGateway
from threading import Lock, Thread
from constant import MAX_PRICE_NUM


from constant import (
    Direction,
    Exchange,
    Product,
    Status,
    OrderType
)

from object import (
    TickData,
    OrderData,
    TradeData,
    AccountData,
    ContractData,
    OrderRequest,
    CancelRequest,
    SubscribeRequest
)

from function import getNoUnderLowerSymbol , getFormatLowerSymbol


REST_HOST = "https://api.huobipro.com"
WEBSOCKET_DATA_HOST = "wss://api.huobi.pro/ws"       # Market Data
#WEBSOCKET_DATA_HOST = "wss://api-aws.huobi.pro/ws"   # Market Data
WEBSOCKET_TRADE_HOST = "wss://api.huobi.pro/ws/v1"     # Account and Order

STATUS_HUOBI2VT = {
    "submitted": Status.NOTTRADED,
    "partial-filled": Status.PARTTRADED,
    "filled": Status.ALLTRADED,
    "cancelling": Status.CANCELLED,
    "partial-canceled": Status.CANCELLED,
    "canceled": Status.CANCELLED,
}

ORDERTYPE_VT2HUOBI = {
    (Direction.LONG, OrderType.MARKET): "buy-market",
    (Direction.SHORT, OrderType.MARKET): "sell-market",
    (Direction.LONG, OrderType.LIMIT): "buy-limit",
    (Direction.SHORT, OrderType.LIMIT): "sell-limit",
}
ORDERTYPE_HUOBI2VT = {v: k for k, v in ORDERTYPE_VT2HUOBI.items()}


huobi_symbols = set()
symbol_name_map = {}

class HuobiGateway(BaseGateway):
    """
    VN Trader Gateway for Huobi connection.
    """

    default_setting = {
        "API Key": "",
        "Secret Key": "",
        "会话数": 3,
        "代理地址": "",
        "代理端口": "",
    }

    exchanges = [Exchange.HUOBI]

    def __init__(self, event_engine):
        """Constructor"""
        super(HuobiGateway,self).__init__(event_engine, "HUOBI")

        self.order_manager = LocalOrderManager(self)

        self.rest_api = HuobiRestApi(self)
        self.trade_ws_api = HuobiTradeWebsocketApi(self)
        self.market_ws_api = HuobiDataWebsocketApi(self)

    def connect(self, setting):
        """"""
        key = setting["API Key"]
        secret = setting["Secret Key"]
        session_number = setting["会话数"]
        proxy_host = setting["代理地址"]
        proxy_port = setting["代理端口"]

        if proxy_port.isdigit():
            proxy_port = int(proxy_port)
        else:
            proxy_port = 0

        self.rest_api.connect(key, secret, session_number,
                              proxy_host, proxy_port)
        self.trade_ws_api.connect(key, secret, proxy_host, proxy_port)
        self.market_ws_api.connect(key, secret, proxy_host, proxy_port)

        self.init_query()

    def subscribe(self, req):
        """"""
        self.market_ws_api.subscribe(req)
        self.trade_ws_api.subscribe(req)

    def send_order(self, req):
        """"""
        return self.rest_api.send_order(req)

    def cancel_order(self, req):
        """"""
        self.rest_api.cancel_order(req)

    def query_account(self):
        """"""
        self.rest_api.query_account_balance()

    def query_position(self):
        """"""
        pass

    def close(self):
        """"""
        self.rest_api.stop()
        self.trade_ws_api.stop()
        self.market_ws_api.stop()

    def process_timer_event(self, event):
        """"""
        self.count += 1
        if self.count < 3:
            return

        self.query_account()

    def init_query(self):
        """"""
        self.count = 0
        self.event_engine.register(EVENT_TIMER, self.process_timer_event)


class HuobiRestApi(RestClient):
    """
    HUOBI REST API
    """

    def __init__(self, gateway):
        """"""
        super().__init__()

        self.gateway = gateway
        self.gateway_name = gateway.gateway_name
        self.order_manager = gateway.order_manager

        self.host = ""
        self.key = ""
        self.secret = ""
        self.account_id = ""

    def sign(self, request):
        """
        Generate HUOBI signature.
        """
        request.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36"
        }
        params_with_signature = create_signature(
            self.key,
            request.method,
            self.host,
            request.path,
            self.secret,
            request.params
        )
        request.params = params_with_signature

        if request.method == "POST":
            request.headers["Content-Type"] = "application/json"

            if request.data:
                request.data = json.dumps(request.data)

        return request

    def connect(
        self,
        key,
        secret,
        session_number,
        proxy_host,
        proxy_port
    ):
        """
        Initialize connection to REST server.
        """
        self.key = key
        self.secret = secret

        self.host, _ = _split_url(REST_HOST)

        self.init(REST_HOST, proxy_host, proxy_port)
        self.start(session_number)

        self.gateway.write_log("REST API启动成功")

        self.query_contract()
        self.query_account()
        self.query_order()

    def query_account(self):
        """"""
        self.add_request(
            method="GET",
            path="/v1/account/accounts",
            callback=self.on_query_account
        )

    def query_account_balance(self):
        """"""
        #path = f"/v1/account/accounts/{self.account_id}/balance"
        path = "/v1/account/accounts/" + str(self.account_id) + "/balance"
        self.add_request(
            method="GET",
            path=path,
            callback=self.on_query_account_balance
        )

    def query_order(self):
        """"""
        self.add_request(
            method="GET",
            path="/v1/order/openOrders",
            callback=self.on_query_order
        )

    def query_contract(self):
        """"""
        self.add_request(
            method="GET",
            path="/v1/common/symbols",
            callback=self.on_query_contract
        )

    def send_order(self, req):
        """"""
        huobi_type = ORDERTYPE_VT2HUOBI.get(
            (req.direction, req.type), ""
        )

        local_orderid = self.order_manager.new_local_orderid()
        order = req.create_order_data(
            local_orderid,
            self.gateway_name
        )
        order.time = datetime.now().strftime("%H:%M:%S")

        data = {
            "account-id": self.account_id,
            "amount": str(req.volume),
            "symbol": req.symbol,
            "type": huobi_type,
            "price": str(req.price),
            "source": "api"
        }

        self.add_request(
            method="POST",
            path="/v1/order/orders/place",
            callback=self.on_send_order,
            data=data,
            extra=order,
            on_error=self.on_send_order_error,
            on_failed=self.on_send_order_failed
        )

        self.order_manager.on_order(order)
        return order.vt_orderid

    def cancel_order(self, req):
        """"""
        sys_orderid = self.order_manager.get_sys_orderid(req.orderid)

        #path = f"/v1/order/orders/{sys_orderid}/submitcancel"
        path = "/v1/order/orders/" + str(sys_orderid) + "/submitcancel"
        self.add_request(
            method="POST",
            path=path,
            callback=self.on_cancel_order,
            extra=req
        )

    def on_query_account(self, data, request):
        """"""
        if self.check_error(data, "查询账户"):
            return

        for d in data["data"]:
            if d["type"] == "spot":
                self.account_id = d["id"]
                #self.gateway.write_log(f"账户代码{self.account_id}查询成功")
                self.gateway.write_log("account_id:" + str(self.account_id) + " query successily!")

        self.query_account_balance()

    def on_query_account_balance(self, data, request):
        """"""
        if self.check_error(data, "查询账户资金"):
            return

        buf = {}
        for d in data["data"]["list"]:
            currency = d["currency"]
            currency_data = buf.setdefault(currency, {})
            currency_data[d["type"]] = float(d["balance"])

        for currency, currency_data in buf.items():
            account = AccountData(
                accountid=currency,
                balance=currency_data["trade"] + currency_data["frozen"],
                frozen=currency_data["frozen"],
                gateway_name=self.gateway_name,
            )

            if account.balance:
                self.gateway.on_account(account)

    def on_query_order(self, data, request):
        """"""
        if self.check_error(data, "查询委托"):
            return

        for d in data["data"]:
            sys_orderid = d["id"]
            local_orderid = self.order_manager.get_local_orderid(sys_orderid)

            direction, order_type = ORDERTYPE_HUOBI2VT[d["type"]]
            dt = datetime.fromtimestamp(d["created-at"] / 1000)
            time = dt.strftime("%H:%M:%S")

            order = OrderData(
                orderid=local_orderid,
                symbol=d["symbol"],
                exchange=Exchange.HUOBI,
                price=float(d["price"]),
                volume=float(d["amount"]),
                type=order_type,
                direction=direction,
                traded=float(d["filled-amount"]),
                status=STATUS_HUOBI2VT.get(d["state"], None),
                time=time,
                gateway_name=self.gateway_name,
            )

            self.order_manager.on_order(order)

        self.gateway.write_log("委托信息查询成功")

    def on_query_contract(self, data, request):  # type: (dict, Request)->None
        """"""
        if self.check_error(data, "查询合约"):
            return

        for d in data["data"]:
            base_currency = d["base-currency"]
            quote_currency = d["quote-currency"]
            #name = f"{base_currency.upper()}/{quote_currency.upper()}"
            name = str(base_currency.upper()) + "/" + str(quote_currency.upper())
            pricetick = 1 / pow(10, d["price-precision"])
            min_volume = 1 / pow(10, d["amount-precision"])

            contract = ContractData(
                symbol=d["symbol"],
                exchange=Exchange.HUOBI,
                name=name,
                pricetick=pricetick,
                size=1,
                min_volume=min_volume,
                product=Product.SPOT,
                gateway_name=self.gateway_name,
            )
            self.gateway.on_contract(contract)

            huobi_symbols.add(contract.symbol)
            symbol_name_map[contract.symbol] = contract.name

        self.gateway.write_log("合约信息查询成功")

    def on_send_order(self, data, request):
        """"""
        order = request.extra

        if self.check_error(data, "委托"):
            order.status = Status.REJECTED
            self.order_manager.on_order(order)
            return

        sys_orderid = data["data"]
        self.order_manager.update_orderid_map(order.orderid, sys_orderid)

    def on_send_order_failed(self, status_code, request):
        """
        Callback when sending order failed on server.
        """
        order = request.extra
        order.status = Status.REJECTED
        self.gateway.on_order(order)

        #msg = f"委托失败，状态码：{status_code}，信息：{request.response.text}"
        msg = "委托失败，状态码：" + str(status_code) + "，信息：" + str(request.response.text)
        self.gateway.write_log(msg)

    def on_send_order_error(
        self, exception_type, exception_value, tb, request
    ):
        """
        Callback when sending order caused exception.
        """
        order = request.extra
        order.status = Status.REJECTED
        self.gateway.on_order(order)

        # Record exception if not ConnectionError
        if not issubclass(exception_type, ConnectionError):
            self.on_error(exception_type, exception_value, tb, request)

    def on_cancel_order(self, data, request):
        """"""
        cancel_request = request.extra
        local_orderid = cancel_request.orderid
        order = self.order_manager.get_order_with_local_orderid(local_orderid)

        if self.check_error(data, "撤单"):
            order.status = Status.REJECTED
        else:
            order.status = Status.CANCELLED
            self.gateway.write_log("委托撤单成功：{}".format(str(order.orderid)))

        self.order_manager.on_order(order)

    def on_error(
        self, exception_type , exception_value , tb, request
    ):
        """
        Callback to handler request exception.
        """
        msg = "触发异常，状态码：{}，信息：{}".format(str(exception_type),str(exception_value))
        self.gateway.write_log(msg)

        sys.stderr.write(
            self.exception_detail(exception_type, exception_value, tb, request)
        )

    def check_error(self, data, func = ""):
        """"""
        if data["status"] != "error":
            return False

        error_code = data["err-code"]
        error_msg = data["err-msg"]

        self.gateway.write_log("{}请求出错，代码：{}，信息：{}".format(str(func),str(error_code),str(error_msg)))
        return True


class HuobiWebsocketApiBase(WebsocketClient):
    """"""
    def __init__(self, gateway):
        """"""
        super(HuobiWebsocketApiBase,self).__init__()

        self.gateway = gateway
        self.gateway_name = gateway.gateway_name

        self.key = ""
        self.secret = ""
        self.sign_host = ""
        self.path = ""

    def connect(
        self,
        key,
        secret,
        url,
        proxy_host,
        proxy_port
    ):
        """"""
        self.key = key
        self.secret = secret

        host, path = _split_url(url)
        self.sign_host = host
        self.path = path

        print(url, proxy_host, proxy_port)

        self.init(url, proxy_host, proxy_port)
        self.start()

    def login(self):
        """"""
        params = {"op": "auth"}
        params.update(create_signature(self.key, "GET", self.sign_host, self.path, self.secret))
        return self.send_packet(params)

    def on_login(self, packet):
        """"""
        pass

    @staticmethod
    def unpack_data(data):
        """"""
        return json.loads(zlib.decompress(data, 31))

    def on_packet(self, packet):
        """"""
        if "ping" in packet:
            req = {"pong": packet["ping"]}
            self.send_packet(req)
        elif "op" in packet and packet["op"] == "ping":
            req = {
                "op": "pong",
                "ts": packet["ts"]
            }
            self.send_packet(req)
        elif "err-msg" in packet:
            return self.on_error_msg(packet)
        elif "op" in packet and packet["op"] == "auth":
            return self.on_login()
        else:
            self.on_data(packet)

    def on_data(self, packet):
        """"""
        print("data : {}".format(packet))

    def on_error_msg(self, packet):
        """"""
        msg = packet["err-msg"]
        if msg == "invalid pong":
            return

        self.gateway.write_log(packet["err-msg"])


class HuobiTradeWebsocketApi(HuobiWebsocketApiBase):
    """"""
    def __init__(self, gateway):
        """"""
        super().__init__(gateway)

        self.order_manager = gateway.order_manager
        self.order_manager.push_data_callback = self.on_data

        self.req_id = 0

    def connect(self, key, secret, proxy_host, proxy_port):
        """"""
        super().connect(key, secret, WEBSOCKET_TRADE_HOST, proxy_host, proxy_port)

    def subscribe(self, req):
        """"""
        self.req_id += 1
        req = {
            "op": "sub",
            "cid": str(self.req_id),
            "topic": "orders.{}".format(req.symbol)
        }
        self.send_packet(req)

    def on_connected(self):
        """"""
        self.gateway.write_log("交易Websocket API连接成功")
        self.login()

    def on_login(self):
        """"""
        self.gateway.write_log("交易Websocket API登录成功")

    def on_data(self, packet):  # type: (dict)->None
        """"""
        op = packet.get("op", None)
        if op != "notify":
            return

        topic = packet["topic"]
        if "orders" in topic:
            self.on_order(packet["data"])

    def on_order(self, data):
        """"""
        sys_orderid = str(data["order-id"])

        order = self.order_manager.get_order_with_sys_orderid(sys_orderid)
        if not order:
            self.order_manager.add_push_data(sys_orderid, data)
            return

        traded_volume = float(data["filled-amount"])

        # Push order event
        order.traded += traded_volume
        order.status = STATUS_HUOBI2VT.get(data["order-state"], None)
        self.order_manager.on_order(order)

        # Push trade event
        if not traded_volume:
            return

        trade = TradeData(
            symbol=order.symbol,
            exchange=Exchange.HUOBI,
            orderid=order.orderid,
            tradeid=str(data["seq-id"]),
            direction=order.direction,
            price=float(data["price"]),
            volume=float(data["filled-amount"]),
            time=datetime.now().strftime("%H:%M:%S"),
            gateway_name=self.gateway_name,
        )
        self.gateway.on_trade(trade)

class HuobiDataWebsocketApi(HuobiWebsocketApiBase):
    """"""

    def __init__(self, gateway):
        """"""
        super(HuobiDataWebsocketApi , self).__init__(gateway)

        self.req_id = 0
        self.ticks = {}

    def connect(self, key, secret, proxy_host, proxy_port):
        """"""
        super(HuobiDataWebsocketApi,self).connect(key, secret, WEBSOCKET_DATA_HOST, proxy_host, proxy_port)

    def on_connected(self):
        """"""
        self.gateway.write_log("行情Websocket API连接成功")

    def subscribe(self, req):
        """"""
        symbol = req.symbol

        # Create tick data buffer
        # tick = TickData(
        #     symbol=symbol,
        #     name=symbol_name_map.get(symbol, ""),
        #     exchange=Exchange.HUOBI,
        #     datetime=datetime.now(),
        #     gateway_name=self.gateway_name,
        # )
        tick = TickData()
        tick.symbol = symbol 
        tick.name = symbol_name_map.get(symbol, "")
        tick.exchange = Exchange.HUOBI
        tick.datetime = datetime.now()
        tick.gateway_name = self.gateway_name
        self.ticks[symbol] = tick

        # Subscribe to market depth update
        self.req_id += 1
        req = {
            "sub": "market." + getNoUnderLowerSymbol(str(symbol)) + ".depth.step0",
            #"sub": f"market.{symbol}.depth.step0",
            "id": str(self.req_id)
        }

        self.send_packet(req)

        # Subscribe to market detail update
        self.req_id += 1
        req = {
            #"sub": f"market.{symbol}.detail",
            "sub": "market." + getNoUnderLowerSymbol(str(symbol)) + ".detail",
            "id": str(self.req_id)
        }
        self.send_packet(req)

    def on_data(self, packet):  # type: (dict)->None
        """"""
        channel = packet.get("ch", None)
        if channel:
            if "depth.step" in channel:
                self.on_market_depth(packet)
            elif "detail" in channel:
                self.on_market_detail(packet)
        elif "err-code" in packet:
            code = packet["err-code"]
            msg = packet["err-msg"]
            #self.gateway.write_log(f"错误代码：{code}, 错误信息：{msg}")
            self.gateway.write_log("错误代码："+code+", 错误信息："+ msg )

    def on_market_depth(self, data):
        """行情深度推送 """
        print(data)
        symbol = data["ch"].split(".")[1]
        symbol = getFormatLowerSymbol(symbol)
        tick = self.ticks[symbol]
        tick.datetime = datetime.fromtimestamp(data["ts"] / 1000)

        bids = data["tick"]["bids"]
        max_num = min(MAX_PRICE_NUM, len(bids))
        for n in range(max_num):
            price, volume = bids[n]
            tick.bid_prices[n] = float(price)
            tick.bid_volumes[n] = float(volume)

        asks = data["tick"]["asks"]
        max_num = min(MAX_PRICE_NUM, len(asks))
        for n in range(max_num):
            price, volume = asks[n]
            tick.ask_prices[n] = float(price)
            tick.ask_volumes[n] = float(volume)

        if tick.last_price:
            self.gateway.on_tick(copy(tick))

    def on_market_detail(self, data):
        """市场细节推送"""
        print(data)
        symbol = data["ch"].split(".")[1]
        symbol = getFormatLowerSymbol(symbol)
        tick = self.ticks[symbol]
        tick.datetime = datetime.fromtimestamp(data["ts"] / 1000)

        tick_data = data["tick"]
        tick.open_price = float(tick_data["open"])
        tick.high_price = float(tick_data["high"])
        tick.low_price = float(tick_data["low"])
        tick.last_price = float(tick_data["close"])
        tick.volume = float(tick_data["vol"])

        if tick.bid_prices[0]:
            self.gateway.on_tick(copy(tick))

class HuobiDataRestfulApi(RestClient):
    """
    HUOBI REST API
    """
    def __init__(self, gateway):
        super(HuobiDataRestfulApi,self).__init__()

        self.gateway = gateway
        self.gateway_name = gateway.gateway_name

        self.ticks = {}

        self.all_symbols_set = set([])
        self.all_reqest_list = []

        self.start_timer_thread(1)

    def start_timer_thread(self, interval):

        self.loop_interval = interval
        self.active_loop = True
        self._loop_thread = Thread(target=self._run_loop_thread)
        self._loop_thread.start()

    def _run_loop_thread(self):
        print("_run_loop_thread(self)")
        while self.active_loop:
            print("self.active_loop")
            for req in self.all_reqest_list:
                print("self.query_depth({})".format(req.symbol))
                self.query_depth(req.symbol)

            time.sleep(self.loop_interval)

    def connect(self,
        key,
        secret,
        session_number,
        proxy_host = "",
        proxy_port = 0
    ):
        self.host, _ = _split_url(REST_HOST)
        print(self.host)
        self.init(REST_HOST, proxy_host, proxy_port)
        print(REST_HOST, proxy_host, proxy_port)
        self.start(session_number)
        print(session_number)

        self.gateway.write_log("REST DATA API启动成功")
        print("REST DATA API启动成功")

    def subscribe(self, req):
        print("subscribe(self, req)")
        symbol = req.symbol
        if symbol not in self.all_symbols_set:

            tick = TickData()
            tick.symbol = symbol 
            tick.name = symbol_name_map.get(symbol, "")
            tick.exchange = Exchange.HUOBI
            tick.datetime = datetime.now()
            tick.gateway_name = self.gateway_name
            self.ticks[symbol] = tick

            self.all_symbols_set.add( symbol )
            self.all_reqest_list.append(req)


    def query_depth(self , symbol , depth = 10 , Type="step0"):
        """"""
        self.add_request(
            method="GET",
            path="/market/depth?symbol={}&type={}&depth={}".format(getNoUnderLowerSymbol(symbol),str(Type),str(depth)),
            callback=self.on_query_depth
        )


    def on_query_depth(self, data, request):
        """"""
        print(data)
        if self.check_error(data , "查询行情"):
            return

        symbol = data["ch"].split(".")[1]
        symbol = getFormatLowerSymbol(symbol)

        tick = self.ticks[symbol]
        tick.datetime = datetime.fromtimestamp(data["ts"] / 1000)

        bids = data["tick"]["bids"]
        max_num = min(MAX_PRICE_NUM, len(bids))
        for n in range(max_num):
            price, volume = bids[n]
            tick.bid_prices[n] = float(price)
            tick.bid_volumes[n] = float(volume)

        asks = data["tick"]["asks"]
        max_num = min(MAX_PRICE_NUM, len(asks))
        for n in range(max_num):
            price, volume = asks[n]
            tick.ask_prices[n] = float(price)
            tick.ask_volumes[n] = float(volume)

        tick.last_price = (tick.ask_prices[0] + tick.bid_prices[0])/2.0

        self.gateway.on_rest_tick(copy(tick))

    def check_error(self, data, func = ""):
        """"""
        if data["status"] != "error":
            return False

        error_code = data["err-code"]
        error_msg = data["err-msg"]

        self.gateway.write_log("{}请求出错，代码：{}，信息：{}".format(str(func),str(error_code),str(error_msg)))
        return True


def _split_url(url):
    """
    将url拆分为host和path
    :return: host, path
    """
    result = re.match("\w+://([^/]*)(.*)", url)  # noqa
    if result:
        return result.group(1), result.group(2)


def create_signature(api_key, method, host, path, secret_key, get_params=None):
    """
    创建签名
    :param get_params: dict 使用GET方法时附带的额外参数(urlparams)
    :return:
    """
    sorted_params = [
        ("AccessKeyId", api_key),
        ("SignatureMethod", "HmacSHA256"),
        ("SignatureVersion", "2"),
        ("Timestamp", datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"))
    ]

    if get_params:
        sorted_params.extend(list(get_params.items()))
        sorted_params = list(sorted(sorted_params))
    encode_params = urllib.parse.urlencode(sorted_params)

    payload = [method, host, path, encode_params]
    payload = "\n".join(payload)
    payload = payload.encode(encoding="UTF8")

    secret_key = secret_key.encode(encoding="UTF8")

    digest = hmac.new(secret_key, payload, digestmod=hashlib.sha256).digest()
    signature = base64.b64encode(digest)

    params = dict(sorted_params)
    params["Signature"] = signature.decode("UTF8")
    return params



if __name__ == "__main__":
    import ssl                                                                                                                                                                                             
    ssl.match_hostname = lambda cert, hostname: hostname == cert['subjectAltName'][0][1]

    import ssl
    ssl.match_hostname = lambda cert, hostname: True

    from event import EventEngine , Event
    from gateway import RedisRabbitmqGateway
    import time
    # test websocketData
    # e = EventEngine()
    # g = RedisRabbitmqGateway(e , "test")

    # huobi = HuobiDataWebsocketApi(g)
    # huobi.connect("c018457f-59e8d348-b1rkuf4drg-dd177","19c55b23-f26f92ab-56666384-2d1ae","",0)

    # time.sleep(10)
    # sub = SubscribeRequest()
    # sub.symbol = "btc_usdt"
    # sub.exchange = "HUOBI"
    # sub.vt_symbol = "btc_usdt.HUOBI"

    # huobi.subscribe( sub )


    # test restfulData
    e = EventEngine()
    g = RedisRabbitmqGateway(e , "test")
    huobi_rest_data = HuobiDataRestfulApi(g)
    huobi_rest_data.connect("","",3)

    sub = SubscribeRequest()
    sub.symbol = "btc_usdt"
    sub.exchange = "HUOBI"
    sub.vt_symbol = "btc_usdt.HUOBI"

    huobi_rest_data.subscribe( sub )


    input()




    


# coding=utf-8

from enum import Enum

MAX_PRICE_NUM = 10
EMPTY_STRING = ""
EMPTY_INT = 0
EMPTY_FLOAT = 0.0
EMPTY_UNICODE = ""

class Direction(Enum):
    """
    Direction of order/trade/position.
    """
    LONG = "多"
    SHORT = "空"
    NET = "净"


class Offset(Enum):
    """
    Offset of order/trade.
    """
    NONE = ""
    OPEN = "开"
    CLOSE = "平"
    CLOSETODAY = "平今"
    CLOSEYESTERDAY = "平昨"


class Status(Enum):
    """
    Order status.
    """
    SUBMITTING = "提交中"
    NOTTRADED = "未成交"
    PARTTRADED = "部分成交"
    ALLTRADED = "全部成交"
    CANCELLED = "已撤销"
    REJECTED = "拒单"


class Product(Enum):
    """
    Product class.
    """
    EQUITY = "股票"
    FUTURES = "期货"
    OPTION = "期权"
    INDEX = "指数"
    FOREX = "外汇"
    SPOT = "现货"
    ETF = "ETF"
    BOND = "债券"
    WARRANT = "权证"
    SPREAD = "价差"
    FUND = "基金"


class OrderType(Enum):
    """
    Order type.
    """
    LIMIT = "限价"
    MARKET = "市价"
    STOP = "STOP"
    FAK = "FAK"
    FOK = "FOK"


class OptionType(Enum):
    """
    Option type.
    """
    CALL = "看涨期权"
    PUT = "看跌期权"


class Exchange(Enum):
    """
    Exchange.
    """
    # Chinese
    CFFEX = "CFFEX"         # China Financial Futures Exchange
    SHFE = "SHFE"           # Shanghai Futures Exchange
    CZCE = "CZCE"           # Zhengzhou Commodity Exchange
    DCE = "DCE"             # Dalian Commodity Exchange
    INE = "INE"             # Shanghai International Energy Exchange
    SSE = "SSE"             # Shanghai Stock Exchange
    SZSE = "SZSE"           # Shenzhen Stock Exchange
    SGE = "SGE"             # Shanghai Gold Exchange
    WXE = "WXE"             # Wuxi Steel Exchange

    # Global
    SMART = "SMART"         # Smart Router for US stocks
    NYMEX = "NYMEX"         # New York Mercantile Exchange
    COMEX = "COMEX"         # a division of theNew York Mercantile Exchange
    GLOBEX = "GLOBEX"       # Globex of CME
    IDEALPRO = "IDEALPRO"   # Forex ECN of Interactive Brokers
    CME = "CME"             # Chicago Mercantile Exchange
    ICE = "ICE"             # Intercontinental Exchange
    SEHK = "SEHK"           # Stock Exchange of Hong Kong
    HKFE = "HKFE"           # Hong Kong Futures Exchange
    SGX = "SGX"             # Singapore Global Exchange
    CBOT = "CBT"            # Chicago Board of Trade
    CBOE = "CBOE"           # Chicago Board Options Exchange
    CFE = "CFE"             # CBOE Futures Exchange
    DME = "DME"             # Dubai Mercantile Exchange
    EUREX = "EUX"           # Eurex Exchange
    APEX = "APEX"           # Asia Pacific Exchange
    LME = "LME"             # London Metal Exchange
    BMD = "BMD"             # Bursa Malaysia Derivatives
    TOCOM = "TOCOM"         # Tokyo Commodity Exchange
    EUNX = "EUNX"           # Euronext Exchange
    KRX = "KRX"             # Korean Exchange

    # CryptoCurrency
    BITMEX = "BITMEX"
    OKEX = "OKEX"
    HUOBI = "HUOBI"
    BITFINEX = "BITFINEX"
    BINANCE = "BINANCE"
    COINBASE = "COINBASE"

    # Special Function
    LOCAL = "LOCAL"         # For local generated data


class Currency(Enum):
    """
    Currency.
    """
    USD = "USD"
    HKD = "HKD"
    CNY = "CNY"


class Interval(Enum):
    """
    Interval of bar data.
    """
    MINUTE = "1m"
    HOUR = "1h"
    DAILY = "d"
    WEEKLY = "w"



# coding=utf-8

import redis

def test1():
    r = redis.Redis(host='localhost', port=6379, decode_responses=True)
    r.set("a","b")
    r.get("a")

    r.hset("h1","r","s")
    r.hget("h2","r","s")

    r.hincrby("hash1", "k3", amount=-1)
    print(r.hgetall("hash1"))

    r.hset("hash1", "k5", "1.0")

    r.lpush("list1", 11, 22, 33)

    r.rpush("list2", 44, 55, 66)    # 在列表的右边，依次添加44,55,66

    print(r.llen("list2"))  # 列表长度
    
    print(r.lrange("list2", 0, -1)) # 切片取出值，范围是索引号0到-1(最后一个元素)



if __name__ == "__main__":
    test1()


# coding=utf-8

from constant import EMPTY_STRING , EMPTY_INT , EMPTY_FLOAT, EMPTY_UNICODE , MAX_PRICE_NUM
from constant import Direction, Exchange, Interval, Offset, Status, Product, OptionType, OrderType

import time
from logging import INFO

class TickData(object):
    """
    Tick data contains information about:
        * last trade in market
        * orderbook snapshot
        * intraday market statistics.
    """

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        # 代码相关
        self.symbol = EMPTY_STRING              # 合约代码
        self.exchange = EMPTY_STRING            # 交易所代码
        self.vt_symbol = EMPTY_STRING            # 合约在vt系统中的唯一代码，通常是 合约代码.交易所代码

        self.name = EMPTY_STRING                # 合约名字
        self.gateway_name = EMPTY_STRING        # 网关名
        
        # 成交数据
        self.last_price = EMPTY_FLOAT            # 最新成交价
        self.last_volume = EMPTY_INT             # 最新成交量
        self.volume = EMPTY_INT                 # 今天总成交量
        self.open_interest = EMPTY_INT           # 持仓量
        self.time = EMPTY_STRING                # 时间 11:20:56.5
        self.date = EMPTY_STRING                # 日期 20151009
        self.datetime = None                    # python的datetime时间对象
        
        # 常规行情
        self.open_price = EMPTY_FLOAT            # 今日开盘价
        self.high_price = EMPTY_FLOAT            # 今日最高价
        self.low_price = EMPTY_FLOAT             # 今日最低价
        self.pre_close_price = EMPTY_FLOAT
        
        self.upperLimit = EMPTY_FLOAT           # 涨停价
        self.lowerLimit = EMPTY_FLOAT           # 跌停价
        
        # 五档行情
        self.bid_prices = [0.0] * MAX_PRICE_NUM
        self.ask_prices = [0.0] * MAX_PRICE_NUM

        self.bid_volumes = [0.0] * MAX_PRICE_NUM
        self.ask_volumes = [0.0] * MAX_PRICE_NUM


class BarData():
    """
    Candlestick bar data of a certain trading period.
    """

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        self.vt_symbol = EMPTY_STRING        # vt系统代码
        self.symbol = EMPTY_STRING          # 代码
        self.exchange = EMPTY_STRING        # 交易所
    
        self.open = EMPTY_FLOAT             # OHLC
        self.high = EMPTY_FLOAT
        self.low = EMPTY_FLOAT
        self.close = EMPTY_FLOAT
        
        self.date = EMPTY_STRING            # bar开始的时间，日期
        self.time = EMPTY_STRING            # 时间
        self.datetime = None                # python的datetime时间对象
        
        self.volume = EMPTY_INT             # 成交量
        self.open_interest = EMPTY_INT       # 持仓量

        self.interval = None                # 时间周期    


class OrderData(object):
    """
    Order data contains information for tracking lastest status
    of a specific order.
    """

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        super(VtOrderData, self).__init__()
        
        # 代码编号相关
        self.symbol = EMPTY_STRING              # 合约代码
        self.exchange = EMPTY_STRING            # 交易所代码
        self.vt_symbol = EMPTY_STRING            # 合约在vt系统中的唯一代码，通常是 合约代码.交易所代码
        
        self.orderid = EMPTY_STRING             # 订单编号
        self.vt_orderid = EMPTY_STRING           # 订单在vt系统中的唯一编号，通常是 Gateway名.订单编号
        
        # 报单相关
        self.direction = EMPTY_UNICODE          # 报单方向
        self.offset = EMPTY_UNICODE             # 报单开平仓
        self.price = EMPTY_FLOAT                # 报单价格
        self.totalVolume = EMPTY_INT            # 报单总数量
        self.tradedVolume = EMPTY_INT           # 报单成交数量
        self.status = EMPTY_UNICODE             # 报单状态
        
        self.order_time = EMPTY_STRING           # 发单时间
        self.cancelTime = EMPTY_STRING          # 撤单时间


class TradeData(object):
    """
    Trade data contains information of a fill of an order. One order
    can have several trade fills.
    """

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        # 代码编号相关
        self.symbol = EMPTY_STRING              # 合约代码
        self.exchange = EMPTY_STRING            # 交易所代码
        self.vt_symbol = EMPTY_STRING            # 合约在vt系统中的唯一代码，通常是 合约代码.交易所代码
        
        self.tradeid = EMPTY_STRING             # 成交编号
        self.vt_tradeid = EMPTY_STRING           # 成交在vt系统中的唯一编号，通常是 Gateway名.成交编号
        
        self.orderid = EMPTY_STRING             # 订单编号
        self.vt_orderid = EMPTY_STRING           # 订单在vt系统中的唯一编号，通常是 Gateway名.订单编号
        
        # 成交相关
        self.direction = EMPTY_UNICODE          # 成交方向
        self.offset = EMPTY_UNICODE             # 成交开平仓
        self.price = EMPTY_FLOAT                # 成交价格
        self.volume = EMPTY_INT                 # 成交数量
        self.tradeTime = EMPTY_STRING           # 成交时间



class PositionData(object):
    """
    Positon data is used for tracking each individual position holding.
    """
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        # 代码编号相关
        self.symbol = EMPTY_STRING              # 合约代码
        self.exchange = EMPTY_STRING            # 交易所代码
        self.vt_symbol = EMPTY_STRING            # 合约在vt系统中的唯一代码，合约代码.交易所代码  
        
        # 持仓相关
        self.direction = EMPTY_STRING           # 持仓方向
        self.position = EMPTY_INT               # 持仓量
        self.frozen = EMPTY_INT                 # 冻结数量
        self.price = EMPTY_FLOAT                # 持仓均价
        self.vt_positionid = EMPTY_STRING       # 持仓在vt系统中的唯一代码，通常是vtSymbol.方向
        self.yd_volume = EMPTY_FLOAT            # 昨持仓
        self.pnl = EMPTY_FLOAT                  # 持仓盈亏



class AccountData(object):
    """
    Account data contains information about balance, frozen and
    available.
    """
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        # 账号代码相关
        self.accountid = EMPTY_STRING           # 账户代码
        self.vt_accountid = EMPTY_STRING         # 账户在vt中的唯一代码，通常是 Gateway名.账户代码
        
        # 数值相关
        self.balance = EMPTY_FLOAT              # 账户净值
        self.frozen = EMPTY_FLOAT               # 冻结金额
        self.available = EMPTY_FLOAT            # 可用资金


class LogData(object):
    """
    Log data is used for recording log messages on GUI or in log files.
    """

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        self.time = time.strftime('%X', time.localtime())       # 日志生成时间
        self.msg = EMPTY_UNICODE                                # 日志信息
        self.level = INFO                                       # 日志级别
        self.gateway_name = EMPTY_STRING                        # gateway


class ContractData(object):
    """
    Contract data contains basic information about each contract traded.
    """
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        self.symbol = EMPTY_STRING              # 代码
        self.exchange = EMPTY_STRING            # 交易所代码
        self.vt_symbol = EMPTY_STRING           # 合约在vt系统中的唯一代码，通常是 合约代码.交易所代码
        self.name = EMPTY_UNICODE               # 合约中文名
        
        self.size = EMPTY_INT                   # 合约大小
        self.pricetick = EMPTY_FLOAT            # 合约最小价格TICK

        self.min_volume = 1                     # 最小交易数量
        self.stop_supported = False             # 是否支持stop order
        self.net_position = False               # 是否是净持仓
        self.history_data = False               # 是否支持历时K线数据

        self.option_strike = EMPTY_FLOAT        # 
        self.option_underlying = EMPTY_FLOAT    # vt_symbol of underlying contract
        self.option_type = None                 #
        self.option_expiry = None               # 

        

class SubscribeRequest(object):
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        self.symbol = EMPTY_STRING              # 代码
        self.exchange = EMPTY_STRING            # 交易所

        self.vt_symbol = EMPTY_STRING



class OrderRequest(object):
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        self.symbol = EMPTY_STRING              # 代码
        self.exchange = EMPTY_STRING            # 交易所
        self.vt_symbol = EMPTY_STRING           # VT合约代码

        self.price = EMPTY_FLOAT                # 价格
        self.volume = EMPTY_INT                 # 数量
    
        self.type = EMPTY_STRING                # 价格类型
        self.direction = EMPTY_STRING           # 买卖
        self.offset = EMPTY_STRING              # 开平
        


class CancelRequest:
    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        self.symbol = EMPTY_STRING              # 代码
        self.exchange = EMPTY_STRING            # 交易所
        self.vt_symbol = EMPTY_STRING           # VT合约代码

        self.orderid = EMPTY_STRING             # 报单号






# coding=utf-8


'''
	etc_btc  --> etcbtc
	eth_usdt.GATEWAY --> ethusdt
	ETH_USDT.GATEWAY --> ethusdt
'''
def getNoUnderLowerSymbol(symbol):
	return (symbol.split('.'))[0].replace('_','').lower()

'''
	ethusdt --> eth_usdt
	etcbtc  --> etc_btc
	ethusdt.HUOBI --> eth_usdt
	etcbtc.HUOBI  --> etc_btc
'''
def getFormatLowerSymbol(symbol):
	symbol = symbol.replace('_','')
	symbol = ((symbol.split('.'))[0]).lower()
	if 'usdt' in symbol:
		return symbol[:-4] + "_usdt"
	else:
		return symbol[:-3] + "_" + symbol[-3:]


if __name__ == "__main__":
	print(getNoUnderLowerSymbol("btc_usdt.HUOBI"))


# coding=utf-8

import multiprocessing
from time import sleep
from datetime import datetime, time

from vnpy.ConfigProduce.ClientQuickQuery import *

def get_chazhi( d1 , d2):
	r1 = 0.0
	r2 = 0.0
	r3 = 0.0
	r4 = 0.0
	if d1 and d2:
		#print(d1)
		#print(d2)
		bid1 = d1["bids"][0][0]
		ask1 = d1["asks"][0][0]

		bid2 = d2["bids"][0][0]
		ask2 = d2["asks"][0][0]

		try:
			r1 = (bid1 - ask2) / bid1
			r2 = (bid2 - ask1) / bid2

			r3 = (bid1 - bid2) / bid1
			r4 = (ask1 - ask2) / ask1
		except Exception,ex:
			pass
	return ( r1 , r2 , r3 , r4)

def test():
	s1 = ClientPosPriceQuery.QueryAllBinanceExchangeInfo()
	s2 = ClientPosPriceQuery.QueryAllHuobiExchangeInfo()

	#aa = list(set(s1.keys() + s2.keys()))
	aa = [ x for x in s1.keys() if x in s2.keys()]

	for key in aa:
		d1 = ClientPosPriceQuery.QueryOrderbook(key , EXCHANGE_BINANCE )
		d2 = ClientPosPriceQuery.QueryOrderbook(key , EXCHANGE_HUOBI )

		(r1 , r2 , r3 , r4) = get_chazhi(d1, d2)
		#print( key, r1, r2 , r3, r4)

		if r1 > 0.003 or r2 > 0.003:
			print( "important" , key , r1 , r2 , r3 , r4) 


if __name__ == "__main__":
	#s = ClientPosPriceQuery.QueryAllBinanceExchangeInfo()
	#print(s.keys())

	#u = ClientPosPriceQuery.QueryOrderbook("xrp_usdt" ,EXCHANGE_BINANCE )
	#print(u)

	test()



# coding=utf-8


#####################################
PREFIX = "bbt_"		# 巴比特

#####################################
#redis config
REDIS_SERVER = '127.0.0.1'
REDIS_PORT = 6379

#####################################
#rabbitmq config
MQ_SERVER = "127.0.0.1"				# rabbtimq_server
MQ_PORT = 5672						# rabbitmq_port
MQ_USER = "admin"					# rabbitmq_user
MQ_PASSWD = "admin"					# rabbitmq_password

FANOUT = "fanout"

#####################################
#redis_key:

REDIS_ORDER_BOOK = "order_book"
REDIS_TICKER = "ticker"

#key_type:[REDIS_ORDER_BOOK , REDIS_TICKER]
def getRedisKey(key_type , exchange):
	return PREFIX + key_type + "_" + exchange

#####################################
#rabbitmq_exchange
MQ_DEPTH = "depth"
MQ_RESTFUL_DEPTH = "restful_depth"

#key_type:[MQ_DEPTH]
def getRabbitExchangeKey(key_type , exchange):
	return PREFIX + "exchange_" + key_type + "_" + exchange

#rabbitmq_send_queue
#key_type:[MQ_DEPTH]
def getRabbitSendQueueKey(key_type , exchange):
	return PREFIX + "send_queue_" + key_type + "_" + exchange

#rabbitmq_receive_queue
#key_type:[MQ_DEPTH]
def getRabbitReceiveQueueKey(key_type, exchange , unique_str):
	return PREFIX + "receive_queue_" + key_type + "_" + exchange + unique_str



if __name__ == "__main__":
	# test
	a = getRabbitExchangeKey(MQ_DEPTH , EXCHANGE_HUOBI)
	print(a)
	


# coding=utf-8

import json
import ssl
import sys
import traceback
import socket
from datetime import datetime
from threading import Lock, Thread
from time import sleep

import websocket


class WebsocketClient(object):
    """
    Websocket API

    After creating the client object, use start() to run worker and ping threads.
    The worker thread connects websocket automatically.

    Use stop to stop threads and disconnect websocket before destroying the client
    object (especially when exiting the programme).

    Default serialization format is json.

    Callbacks to overrides:
    * unpack_data
    * on_connected
    * on_disconnected
    * on_packet
    * on_error

    After start() is called, the ping thread will ping server every 60 seconds.

    If you want to send anything other than JSON, override send_packet.
    """

    def __init__(self):
        """Constructor"""
        self.host = None

        self._ws_lock = Lock()
        self._ws = None

        self._worker_thread = None
        self._ping_thread = None
        self._active = False

        self.proxy_host = None
        self.proxy_port = None
        self.ping_interval = 60     # seconds
        self.header = {}

        # For debugging
        self._last_sent_text = None
        self._last_received_text = None

    #def init(self, host: str, proxy_host: str = "", proxy_port: int = 0, ping_interval: int = 60, header: dict = None):
    def init(self, host, proxy_host = "", proxy_port = 0, ping_interval = 60, header = None):
        """
        :param ping_interval: unit: seconds, type: int
        """
        self.host = host
        self.ping_interval = ping_interval  # seconds

        if header:
            self.header = header

        if proxy_host and proxy_port:
            self.proxy_host = proxy_host
            self.proxy_port = proxy_port

    def start(self):
        """
        Start the client and on_connected function is called after webscoket
        is connected succesfully.

        Please don't send packet untill on_connected fucntion is called.
        """

        self._active = True
        self._worker_thread = Thread(target=self._run)
        self._worker_thread.start()

        self._ping_thread = Thread(target=self._run_ping)
        self._ping_thread.start()

    def stop(self):
        """
        Stop the client.
        """
        self._active = False
        self._disconnect()

    def join(self):
        """
        Wait till all threads finish.

        This function cannot be called from worker thread or callback function.
        """
        self._ping_thread.join()
        self._worker_thread.join()

    def send_packet(self, packet):
        """
        Send a packet (dict data) to server

        override this if you want to send non-json packet
        """
        text = json.dumps(packet)
        print(text)
        self._record_last_sent_text(text)
        return self._send_text(text)

    def _send_text(self, text):
        """
        Send a text string to server.
        """
        ws = self._ws
        print("_send_text")
        if ws:
            print("true")
            ws.send(text, opcode=websocket.ABNF.OPCODE_TEXT)

    def _send_binary(self, data):
        """
        Send bytes data to server.
        """
        ws = self._ws
        if ws:
            ws._send_binary(data)

    def _create_connection(self, *args, **kwargs):
        """"""
        return websocket.create_connection(*args, **kwargs)

    def _ensure_connection(self):
        """"""
        print("_ensure_connection")
        triggered = False
        with self._ws_lock:
            if self._ws is None:
                print("self._ws = self._create_connection")
                # self._ws = self._create_connection(
                #     self.host,
                #     sslopt={"cert_reqs": ssl.CERT_NONE},
                #     http_proxy_host=self.proxy_host,
                #     http_proxy_port=self.proxy_port,
                #     header=self.header
                # )
                self._ws = self._create_connection(
                    self.host,
                    header=self.header,
                    verify=False
                )
                print("triggered")
                triggered = True
        if triggered:
            print("self.on_connected()")
            self.on_connected()

    def _disconnect(self):
        """
        """
        triggered = False
        ws = None
        with self._ws_lock:
            if self._ws:
                #ws: websocket.WebSocket = self._ws
                ws = self._ws
                self._ws = None

                triggered = True
        if triggered:
            if ws:
                ws.close()
            self.on_disconnected()

    def _run(self):
        """
        Keep running till stop is called.
        """
        try:
            while self._active:
                print("self._active")
                try:
                    self._ensure_connection()
                    ws = self._ws
                    if ws:
                        text = ws.recv()

                        # ws object is closed when recv function is blocking
                        if not text:
                            self._disconnect()
                            continue

                        self._record_last_received_text(text)

                        try:
                            data = self.unpack_data(text)
                        except ValueError as e:
                            print("websocket unable to parse data: " + text)
                            raise e

                        self.on_packet(data)
                # ws is closed before recv function is called
                # For socket.error, see Issue #1608
                except (websocket.WebSocketConnectionClosedException, socket.error):
                    self._disconnect()

                # other internal exception raised in on_packet
                except:  # noqa
                    et, ev, tb = sys.exc_info()
                    self.on_error(et, ev, tb)
                    self._disconnect()
        except:  # noqa
            et, ev, tb = sys.exc_info()
            self.on_error(et, ev, tb)
        self._disconnect()

    @staticmethod
    def unpack_data(data):
        """
        Default serialization format is json.

        override this method if you want to use other serialization format.
        """
        return json.loads(data)

    def _run_ping(self):
        """"""
        while self._active:
            try:
                self._ping()
            except:  # noqa
                et, ev, tb = sys.exc_info()
                self.on_error(et, ev, tb)

                # self._run() will reconnect websocket
                sleep(1)

            for i in range(self.ping_interval):
                if not self._active:
                    break
                sleep(1)

    def _ping(self):
        """"""
        ws = self._ws
        if ws:
            ws.send("ping", websocket.ABNF.OPCODE_PING)

    @staticmethod
    def on_connected():
        """
        Callback when websocket is connected successfully.
        """
        print("on_connected")
        #pass

    @staticmethod
    def on_disconnected():
        """
        Callback when websocket connection is lost.
        """
        pass

    @staticmethod
    def on_packet(packet):
        """
        Callback when receiving data from server.
        """
        pass

    def on_error(self, exception_type , exception_value , tb):
        """
        Callback when exception raised.
        """
        sys.stderr.write(
            self.exception_detail(exception_type, exception_value, tb)
        )
        return sys.excepthook(exception_type, exception_value, tb)

    def exception_detail(
        self, exception_type , exception_value, tb
    ):
        """
        Print detailed exception information.
        """
        text = "[{}]: Unhandled WebSocket Error:{}\n".format(
            datetime.now().isoformat(), exception_type
        )
        text += "LastSentText:\n{}\n".format(self._last_sent_text)
        text += "LastReceivedText:\n{}\n".format(self._last_received_text)
        text += "Exception trace: \n"
        text += "".join(
            traceback.format_exception(exception_type, exception_value, tb)
        )
        return text

    def _record_last_sent_text(self, text):
        """
        Record last sent text for debug purpose.
        """
        self._last_sent_text = text[:1000]

    def _record_last_received_text(self, text):
        """
        Record last received text for debug purpose.
        """
        self._last_received_text = text[:1000]



"""
Event-driven framework of vn.py framework.
"""

from collections import defaultdict
from queue import Empty, Queue
from threading import Thread
from time import sleep

EVENT_TIMER = "eTimer"
EVENT_TICK = "eTick."
EVENT_TRADE = "eTrade."
EVENT_ORDER = "eOrder."
EVENT_POSITION = "ePosition."
EVENT_ACCOUNT = "eAccount."
EVENT_CONTRACT = "eContract."
EVENT_LOG = "eLog"


class Event:
    """
    Event object consists of a type string which is used
    by event engine for distributing event, and a data
    object which contains the real data.
    """

    def __init__(self, type, data = None):
        """"""
        self.type = type
        self.data = data


# Defines handler function to be used in event engine.
# HandlerType = Callable[[Event], None]


class EventEngine:
    """
    Event engine distributes event object based on its type
    to those handlers registered.

    It also generates timer event by every interval seconds,
    which can be used for timing purpose.
    """

    def __init__(self, interval = 1):
        """
        Timer event is generated every 1 second by default, if
        interval not specified.
        """
        self._interval = interval
        self._queue = Queue()
        self._active = False
        self._thread = Thread(target=self._run)
        self._timer = Thread(target=self._run_timer)
        self._handlers = defaultdict(list)
        self._general_handlers = []

    def _run(self):
        """
        Get event from queue and then process it.
        """
        while self._active:
            try:
                event = self._queue.get(block=True, timeout=1)
                self._process(event)
            except Empty:
                pass

    def _process(self, event):
        """
        First ditribute event to those handlers registered listening
        to this type.

        Then distrubute event to those general handlers which listens
        to all types.
        """
        if event.type in self._handlers:
            [handler(event) for handler in self._handlers[event.type]]

        if self._general_handlers:
            [handler(event) for handler in self._general_handlers]

    def _run_timer(self):
        """
        Sleep by interval second(s) and then generate a timer event.
        """
        while self._active:
            sleep(self._interval)
            event = Event(EVENT_TIMER)
            self.put(event)

    def start(self):
        """
        Start event engine to process events and generate timer events.
        """
        self._active = True
        self._thread.start()
        self._timer.start()

    def stop(self):
        """
        Stop event engine.
        """
        self._active = False
        self._timer.join()
        self._thread.join()

    def put(self, event):
        """
        Put an event object into event queue.
        """
        self._queue.put(event)

    def register(self, type, handler):
        """
        Register a new handler function for a specific event type. Every
        function can only be registered once for each event type.
        """
        handler_list = self._handlers[type]
        if handler not in handler_list:
            handler_list.append(handler)

    def unregister(self, type, handler):
        """
        Unregister an existing handler function from event engine.
        """
        handler_list = self._handlers[type]

        if handler in handler_list:
            handler_list.remove(handler)

        if not handler_list:
            self._handlers.pop(type)

    def register_general(self, handler):
        """
        Register a new handler function for all event types. Every
        function can only be registered once for each event type.
        """
        if handler not in self._general_handlers:
            self._general_handlers.append(handler)

    def unregister_general(self, handler):
        """
        Unregister an existing general handler function.
        """
        if handler in self._general_handlers:
            self._general_handlers.remove(handler)



# coding=utf-8

import multiprocessing
import os
import sys
import traceback
from datetime import datetime
from enum import Enum
from multiprocessing.dummy import Pool
from threading import Lock

import requests


class RequestStatus(Enum):
    ready = 0  # Request created
    success = 1  # Request successful (status code 2xx)
    failed = 2  # Request failed (status code not 2xx)
    error = 3  # Exception raised


#pool: multiprocessing.pool.Pool = Pool(os.cpu_count() * 20)
#multiprocessing.pool.Pool = Pool(os.cpu_count() * 20)
#pool = Pool(os.cpu_count() * 20)
pool = Pool(20)

class Request(object):
    """
    Request object for status check.
    """

    def __init__(
        self,
        method,
        path,
        params,
        data,
        headers,
        callback = None,
        on_failed = None,
        on_error = None,
        extra = None,
    ):
        """"""
        self.method = method
        self.path = path
        self.callback = callback
        self.params = params
        self.data = data
        self.headers = headers

        self.on_failed = on_failed
        self.on_error = on_error
        self.extra = extra

        self.response = None
        self.status = RequestStatus.ready

    def __str__(self):
        if self.response is None:
            status_code = "terminated"
        else:
            status_code = self.response.status_code

        return (
            "request : {} {} {} because {}: \n"
            "headers: {}\n"
            "params: {}\n"
            "data: {}\n"
            "response:"
            "{}\n".format(
                self.method,
                self.path,
                self.status.name,
                status_code,
                self.headers,
                self.params,
                self.data,
                "" if self.response is None else self.response.text,
            )
        )


class RestClient(object):
    """
    HTTP Client designed for all sorts of trading RESTFul API.

    * Reimplement sign function to add signature function.
    * Reimplement on_failed function to handle Non-2xx responses.
    * Use on_failed parameter in add_request function for individual Non-2xx response handling.
    * Reimplement on_error function to handle exception msg.
    """

    class Session:

        def __init__(self, client , session ):
            self.client = client
            self.session = session

        def __enter__(self):
            return self.session

        def __exit__(self, exc_type, exc_val, exc_tb):
            with self.client._sessions_lock:
                self.client._sessions.append(self.session)

    def __init__(self):
        """
        """
        self.url_base = ''  # type: str
        self._active = False

        self.proxies = None

        self._tasks_lock = Lock()
        #self._tasks: List[multiprocessing.pool.AsyncResult] = []
        self._tasks = []
        self._sessions_lock = Lock()
        #self._sessions: List[requests.Session] = []
        self._sessions = []

    def init(self, url_base , proxy_host = "", proxy_port = 0):
        """
        Init rest client with url_base which is the API root address.
        e.g. 'https://www.bitmex.com/api/v1/'
        """
        self.url_base = url_base

        if proxy_host and proxy_port:
            #proxy = f"{proxy_host}:{proxy_port}"
            proxy = str(proxy_host) + ":" + str(proxy_port)
            self.proxies = {"http": proxy, "https": proxy}

    def _create_session(self):
        """"""
        return requests.session()

    def start(self, n = 3):
        """
        Start rest client with session count n.
        """
        if self._active:
            return
        self._active = True

    def stop(self):
        """
        Stop rest client immediately.
        """
        self._active = False

    def join(self):
        """
        Wait till all requests are processed.
        """
        for task in self._tasks:
            task.wait()

    def add_request(
        self,
        method,
        path,
        callback,
        params = None,
        data = None,
        headers = None,
        on_failed = None,
        on_error = None,
        extra = None,
    ):
        """
        Add a new request.
        :param method: GET, POST, PUT, DELETE, QUERY
        :param path:
        :param callback: callback function if 2xx status, type: (dict, Request)
        :param params: dict for query string
        :param data: Http body. If it is a dict, it will be converted to form-data. Otherwise, it will be converted to bytes.
        :param headers: dict for headers
        :param on_failed: callback function if Non-2xx status, type, type: (code, dict, Request)
        :param on_error: callback function when catching Python exception, type: (etype, evalue, tb, Request)
        :param extra: Any extra data which can be used when handling callback
        :return: Request
        """
        request = Request(
            method=method,
            path=path,
            params=params,
            data=data,
            headers=headers,
            callback=callback,
            on_failed=on_failed,
            on_error=on_error,
            extra=extra,
        )
        task = pool.apply_async(
            self._process_request,
            args=[request, ],
            callback=self._clean_finished_tasks,
            # error_callback=lambda e: self.on_error(type(e), e, e.__traceback__, request),
        )
        self._push_task(task)
        return request

    def _push_task(self, task):
        with self._tasks_lock:
            print("append")
            self._tasks.append(task)

    def _clean_finished_tasks(self, result):
        with self._tasks_lock:
            not_finished_tasks = [i for i in self._tasks if not i.ready()]
            self._tasks = not_finished_tasks

    def _get_session(self):
        with self._sessions_lock:
            if self._sessions:
                print("self.Session(self, self._sessions.pop())")
                return self.Session(self, self._sessions.pop())
            else:
                print("self.Session(self, self._create_session())")
                return self.Session(self, self._create_session())

    def sign(self, request):
        """
        This function is called before sending any request out.
        Please implement signature method here.
        @:return (request)
        """
        return request

    def on_failed(self, status_code, request):
        """
        Default on_failed handler for Non-2xx response.
        """
        sys.stderr.write(str(request))

    def on_error(
        self,
        exception_type,
        exception_value,
        tb,
        request,
    ):
        """
        Default on_error handler for Python exception.
        """
        sys.stderr.write(
            self.exception_detail(exception_type, exception_value, tb, request)
        )
        sys.excepthook(exception_type, exception_value, tb)

    def exception_detail(
        self,
        exception_type,
        exception_value,
        tb,
        request,
    ):
        text = "[{}]: Unhandled RestClient Error:{}\n".format(
            datetime.now().isoformat(), exception_type
        )
        text += "request:{}\n".format(request)
        text += "Exception trace: \n"
        text += "".join(
            traceback.format_exception(exception_type, exception_value, tb)
        )
        return text

    def _process_request(
        self, request
    ):
        """
        Sending request to server and get result.
        """
        try:
            print("_process_request")
            with self._get_session() as session:
                request = self.sign(request)

                url = self.make_full_url(request.path)
                print(url)
                response = session.request(
                    request.method,
                    url,
                    headers=request.headers,
                    params=request.params,
                    data=request.data,
                    proxies=self.proxies,
                    verify=False
                )
                print("request.response")
                request.response = response
                status_code = response.status_code
                print("status_code:{}".format(status_code))
                if status_code // 100 == 2:  # 2xx codes are all successful
                    if status_code == 204:
                        json_body = None
                    else:
                        json_body = response.json()

                    request.callback(json_body, request)
                    request.status = RequestStatus.success
                else:
                    request.status = RequestStatus.failed

                    if request.on_failed:
                        request.on_failed(status_code, request)
                    else:
                        self.on_failed(status_code, request)
        except Exception:
            request.status = RequestStatus.error
            t, v, tb = sys.exc_info()
            if request.on_error:
                request.on_error(t, v, tb, request)
            else:
                self.on_error(t, v, tb, request)

    def make_full_url(self, path):
        """
        Make relative api path into full url.
        eg: make_full_url('/get') == 'http://xxxxx/get'
        """
        url = self.url_base + path
        return url

    def request(
        self,
        method,
        path,
        params = None,
        data = None,
        headers = None,
    ):
        """
        Add a new request.
        :param method: GET, POST, PUT, DELETE, QUERY
        :param path:
        :param params: dict for query string
        :param data: dict for body
        :param headers: dict for headers
        :return: requests.Response
        """
        request = Request(
            method,
            path,
            params,
            data,
            headers
        )
        request = self.sign(request)

        url = self.make_full_url(request.path)

        response = requests.request(
            request.method,
            url,
            headers=request.headers,
            params=request.params,
            data=request.data,
            proxies=self.proxies,
            verify=False
        )
        return response



# coding=utf-8

import redis
import pika

from object import LogData

import cool_trading.config as config
from cool_trading.service import MQSender

from event import (
    EVENT_TICK,
    EVENT_ORDER,
    EVENT_TRADE,
    EVENT_POSITION,
    EVENT_ACCOUNT,
    EVENT_CONTRACT,
    EVENT_LOG,

    Event
)

class BaseGateway(object):
	# Fields required in setting dict for connect function.
    default_setting = {}

    # Exchanges supported in the gateway.
    exchanges = []

    def __init__(self, event_engine , gateway_name ):
        """"""
        self.event_engine = event_engine
        self.gateway_name = gateway_name

    def on_event(self, type , data = None):
        """
        General event push.
        """
        event = Event(type, data)
        self.event_engine.put(event)

    def on_tick(self, tick ):
        """
        Tick event push.
        Tick event of a specific vt_symbol is also pushed.
        """
        print("base on_tick")
        self.on_event(EVENT_TICK, tick)
        self.on_event(EVENT_TICK + tick.vt_symbol, tick)

    def on_rest_tick(self,tick):
        print("base on_rest_tick")
        self.on_event(EVENT_TICK, tick)
        self.on_event(EVENT_TICK + tick.vt_symbol, tick)

    def on_trade(self, trade):
        """
        Trade event push.
        Trade event of a specific vt_symbol is also pushed.
        """
        self.on_event(EVENT_TRADE, trade)
        self.on_event(EVENT_TRADE + trade.vt_symbol, trade)

    def on_order(self, order):
        """
        Order event push.
        Order event of a specific vt_orderid is also pushed.
        """
        self.on_event(EVENT_ORDER, order)
        self.on_event(EVENT_ORDER + order.vt_orderid, order)

    def on_position(self, position):
        """
        Position event push.
        Position event of a specific vt_symbol is also pushed.
        """
        self.on_event(EVENT_POSITION, position)
        self.on_event(EVENT_POSITION + position.vt_symbol, position)

    def on_account(self, account):
        """
        Account event push.
        Account event of a specific vt_accountid is also pushed.
        """
        self.on_event(EVENT_ACCOUNT, account)
        self.on_event(EVENT_ACCOUNT + account.vt_accountid, account)

    def on_log(self, log):
        """
        Log event push.
        """
        self.on_event(EVENT_LOG, log)

    def on_contract(self, contract):
        """
        Contract event push.
        """
        self.on_event(EVENT_CONTRACT, contract)

    def write_log(self, msg):
        """
        Write a log event from gateway.
        """
        #log = LogData(msg=msg, gateway_name=self.gateway_name)
        log = LogData()
        log.msg = msg
        log.gateway_name = self.gateway_name
        self.on_log(log)

    def connect(self, setting):
        """
        Start gateway connection.

        to implement this method, you must:
        * connect to server if necessary
        * log connected if all necessary connection is established
        * do the following query and response corresponding on_xxxx and write_log
            * contracts : on_contract
            * account asset : on_account
            * account holding: on_position
            * orders of account: on_order
            * trades of account: on_trade
        * if any of query above is failed,  write log.

        future plan:
        response callback/change status instead of write_log

        """
        pass

    def close(self):
        """
        Close gateway connection.
        """
        pass

    def subscribe(self, req):
        """
        Subscribe tick data update.
        """
        pass


    def send_order(self, req):
        """
        Send a new order to server.

        implementation should finish the tasks blow:
        * create an OrderData from req using OrderRequest.create_order_data
        * assign a unique(gateway instance scope) id to OrderData.orderid
        * send request to server
            * if request is sent, OrderData.status should be set to Status.SUBMITTING
            * if request is failed to sent, OrderData.status should be set to Status.REJECTED
        * response on_order:
        * return OrderData.vt_orderid

        :return str vt_orderid for created OrderData
        """
        pass

    def cancel_order(self, req):
        """
        Cancel an existing order.
        implementation should finish the tasks blow:
        * send request to server
        """
        pass

    def send_orders(self, reqs):
        """
        Send a batch of orders to server.
        Use a for loop of send_order function by default.
        Reimplement this function if batch order supported on server.
        """
        vt_orderids = []

        for req in reqs:
            vt_orderid = self.send_order(req)
            vt_orderids.append(vt_orderid)

        return vt_orderids

    def cancel_orders(self, reqs):
        """
        Cancel a batch of orders to server.
        Use a for loop of cancel_order function by default.
        Reimplement this function if batch cancel supported on server.
        """
        for req in reqs:
            self.cancel_order(req)


    def query_account(self):
        """
        Query account balance.
        """
        pass

    def query_position(self):
        """
        Query holding positions.
        """
        pass

    def query_history(self, req):
        """
        Query bar history data.
        """
        pass

    def get_default_setting(self):
        """
        Return default setting dict.
        """
        return self.default_setting

'''
only to record 
'''
class RedisRabbitmqGateway(BaseGateway):
    def __init__(self, event_engine , gateway_name ):
        super(HuobiGateway,self).__init__(event_engine, gateway_name)


    def on_tick(self, tick):
        print("on_tick")
        print(tick.__dict__)
    
    def on_rest_tick(self, tick):
        print("on_rest_tick")
        print(tick.__dict__)



# coding=utf-8

import ccxt
import json

huobipro = ccxt.huobipro()
data = huobipro.fetch_ticker('BTC/USDT')

#print(data)

bid_huobi = data["info"]["bid"][0]
ask_huobi = data["info"]["ask"][0]

print("huobi",bid_huobi,ask_huobi)

binance = ccxt.binance()
#print(binance.fetch_ticker('BTC/USDT'))
bid_binance = float(data["info"]["bid"][0])
ask_binance = float(data["info"]["ask"][0])
print("binance",bid_binance,ask_binance)

okex = ccxt.okex()
#print(okex.fetch_ticker('BTC/USDT'))
data = okex.fetch_ticker('BTC/USDT')
bid_okex = data["info"]["buy"]
ask_okex = data["info"]["sell"]
print("okex",bid_okex,ask_okex)

#print(binance.load_markets())
#print(okex.load_markets())
print(huobipro.load_markets())

# hitbtc   = ccxt.hitbtc({'verbose': True})
# bitmex   = ccxt.bitmex()
# huobipro = ccxt.huobipro()
# exmo     = ccxt.exmo({
#     'apiKey': 'YOUR_PUBLIC_API_KEY',
#     'secret': 'YOUR_SECRET_PRIVATE_KEY',
# })
# kraken = ccxt.kraken({
#     'apiKey': 'YOUR_PUBLIC_API_KEY',
#     'secret': 'YOUR_SECRET_PRIVATE_KEY',
# })

# exchange_id = 'binance'
# exchange_class = getattr(ccxt, exchange_id)
# exchange = exchange_class({
#     'apiKey': 'YOUR_API_KEY',
#     'secret': 'YOUR_SECRET',
#     'timeout': 30000,
#     'enableRateLimit': True,
# })

# hitbtc_markets = hitbtc.load_markets()

# print(hitbtc.id, hitbtc_markets)
# print(bitmex.id, bitmex.load_markets())
# print(huobipro.id, huobipro.load_markets())

# print(hitbtc.fetch_order_book(hitbtc.symbols[0]))
# print(bitmex.fetch_ticker('BTC/USD'))
# print(huobipro.fetch_trades('LTC/CNY'))

# print(exmo.fetch_balance())

# # sell one ฿ for market price and receive $ right now
# print(exmo.id, exmo.create_market_sell_order('BTC/USD', 1))

# # limit buy BTC/EUR, you pay €2500 and receive ฿1  when the order is closed
# print(exmo.id, exmo.create_limit_buy_order('BTC/EUR', 1, 2500.00))

# # pass/redefine custom exchange-specific order params: type, amount, price, flags, etc...
# kraken.create_market_buy_order('BTC/USD', 1, {'trading_agreement': 'agree'})


# coding=utf-8


 
import logging
import time
from queue import Empty, Queue
 
import pika
 
from threading import Thread, Lock, enumerate

import cool_trading.config as config
 
class MQBase(object):
    """ 消息队列基类, 该类线程不安全的 """
 
    def __init__(self, host, port, user, passwd, exchange, exchange_type='fanout', ack=True, persist=True, **kwargs):  # 默认开启手动消息确认, 交换机\队列\消息持久化
        """ 当开启手动消息确认, 要考虑消息重入的情况 """
 
        self._conn = None
        self._channel = None
        self._properties = None
 
        self.host = host
        self.port = port
        self.user = user
        self.passwd = passwd
        self.exchange = exchange
        self.exchange_type = exchange_type
        self.ack = ack
        self.persist = persist

        user_pwd = pika.PlainCredentials('admin', '123456')

        self.mutex = Lock()
 
    def _get_channel(self):
        """ 创建连接与信道, 声明交换机 """
        #print("_get_channel")
        if self._check_alive():
            return
        else:
            self._clear()

        #print("is alive")
 
        if not self._conn:
            #print("self._conn")
            user_pwd = pika.PlainCredentials(self.user, self.passwd)
            self._conn = pika.BlockingConnection(pika.ConnectionParameters(host=self.host,
                                                                           port=self.port,
                                                                           credentials=user_pwd,
                                                                           heartbeat=0))  # 建立连接并关闭心跳检测
            #print("fuck")

        if not self._channel:
            #print("self._channel = self._conn.channel()")
            self._channel = self._conn.channel()  # 建立信道
            #print("what")
            self._channel.exchange_declare(exchange=self.exchange,
                                           exchange_type=self.exchange_type,
                                           durable=self.persist)  # 声明交换机
            #print("abc")
            if self.ack:
                self._channel.confirm_delivery()  # 在该信道中开启消息确认
                #print("kk")
            self._properties = pika.BasicProperties(delivery_mode=(2 if self.persist else 0))  # 消息持久化
            #print("??")
 
    def _clear(self):
        """ 清理连接与信道 """
 
        def clear_conn():
            if self._conn and self._conn.is_open:
                self._conn.close()
            self._conn = None
 
        def clear_channel():
            if self._channel and self._channel.is_open:
                self._channel.close()
            self._channel = None
 
        if not (self._conn and self._conn.is_open):
            clear_conn()  # 清理连接
        clear_channel()  # 清理信道
 
    def _check_alive(self):
        """ 检查连接与信道是否存活 """
        return self._channel and self._channel.is_open and self._conn and self._conn.is_open
 
 
class MQSender(MQBase):
    """ 生产者, 该类是线程不安全的 .. 加锁 """

    def send(self, route, msg):
        def try_send():
            success = True
            try:
                self._get_channel()
                self._channel.basic_publish(exchange=self.exchange,
                                                      routing_key=route,
                                                      body=str(msg),
                                                      properties=self._properties)
            except Exception as e:
                print(e)
                success = False
 
            return success

        self.mutex.acquire()

        ret = try_send() or try_send()
        if not ret:
            self._clear()

        self.mutex.release()
 
        return ret
 
 
class MQReceiver(MQBase):
    """
    消费者, 该类是线程安全的
    通过receive_queue支持多线程, 不直接创建多实例是想减少底层TCP连接
    """
 
    def __init__(self, *args, **kwargs):
        self._has_start = False
        #self._has_start = True
        self.prefetch_count = kwargs.get('prefetch_count', 0)  # 每次只接受prefetch_count条消息, 处理完再接收新的
        self.receive_queue = Queue(self.prefetch_count)  # 接收缓冲队列, 线程安全的
        super(self.__class__, self).__init__(*args, **kwargs)
 
    def _declare_queue(self, queue_name):
        """ 声明队列 """
        #print("_declare_queue")
        self._channel.queue_declare(queue=queue_name, durable=self.persist , auto_delete = not self.ack)  # 声明队列
        self._channel.queue_bind(queue=queue_name, exchange=self.exchange, routing_key=queue_name)  # 绑定队列到交换机
 
    def _subscribe_queue(self, queue_route):
        """ 订阅队列  """
        #print("_subscribe_queue:{}".format(queue_route))
        try:
            self._declare_queue(queue_route)
            if self.prefetch_count > 0:
                self._channel.basic_qos(prefetch_count=self.prefetch_count)  # 负载均衡
            self._channel.basic_consume(queue_route ,self._handler)  # 订阅队列, 并分配消息处理器
 
        except Exception as e:
            print(e)
            return False
 
        return True
 
    def _handler(self, channel, method_frame, header_frame, body):  # 消息处理器
        """ 收到消息后的回调 """
        #print ('wait message num is {num}'.format(num=channel.get_waiting_message_count()))
        print ("receive:{}".format(str(body)))
        #self.receive_queue.put((body, self._conn, channel, method_frame.delivery_tag))  # 压入队列
        channel.basic_ack(delivery_tag = method_frame.delivery_tag)
 
    def start(self, queue_route = ""):
        """ 开始消费 """
        print("self._has_start queue:{} _has_start:{}".format(queue_route,self._has_start))
        if self._has_start:
            return
        try:
            self._get_channel()
            if self._subscribe_queue(queue_route):
                self._channel.start_consuming()  # 开始消费, 一个连接下仅能调用该方法一次, 否则抛出RecursionError
            self._has_start = True
 
        except Exception as e:
            pass
 
    def _clear(self):
        self.receive_queue = Queue(self.prefetch_count)  # 避免消息重入缓冲队列
        super(self.__class__, self)._clear()
 
 
def test():
    from threading import Thread, current_thread
    from functools import partial
 
    def producer_func():
        for index in range(50):
            print('sender send {content}'.format(content=index))
            print(sender.send('s', index))
 
    def consumer_func():
        receiver.start('test_queue')
        #receiver.start('abc')

    def consumer_func2():
        receiver2.start('abc')
 
    def deal_message(r):
        #print("deal_msg")
        while True:
            msg, conn, channel, ack_tag = r.receive_queue.get()
            #print(msg, conn, channel, ack_tag)
            interval = int(msg) / 5.0
            print ('{thread_name}: get {msg}, wait {interval}s, ack_tag is {delivery_tag}'.format(
                msg=msg, thread_name=current_thread().getName(), interval=interval, delivery_tag=ack_tag))
            if current_thread().getName() == 'handler_1':  # 测试负载均衡
                time.sleep(60)
            else:
                time.sleep(interval)
            conn.add_callback_threadsafe(partial(channel.basic_ack, ack_tag))  # 确认消息成功处理
 
    handler_num = 5
    sender = MQSender(host=config.MQ_SERVER , port=config.MQ_PORT, user=config.MQ_USER, passwd=config.MQ_PASSWD ,exchange='test_exchange')
    receiver = MQReceiver(host=config.MQ_SERVER , port=config.MQ_PORT, user=config.MQ_USER, passwd=config.MQ_PASSWD ,exchange='test_exchange', prefetch_count=handler_num)
    receiver2 = MQReceiver(host=config.MQ_SERVER , port=config.MQ_PORT, user=config.MQ_USER, passwd=config.MQ_PASSWD ,exchange='test_exchange', prefetch_count=handler_num)
 
    Thread(target=consumer_func).start()  # 消费者

    Thread(target=consumer_func2).start()  # 消费者

    Thread(target=producer_func).start()  # 生产者
 
    #for i in range(handler_num):
    #    Thread(target=deal_message, args=(receiver,), name='handler_%d' % i).start()  # 消息处理器
 
 
if __name__ == '__main__':
    test()


# coding:utf-8

from setuptools import Extension, find_packages, setup
# or
# from distutils.core import setup  

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
        name='cool_trading',     # 包名字
        version='1.0.1',           # 包版本
        author="ipqhjjybj",
        author_email='250657661@qq.com',  # 作者邮箱
        license="AGPL v3",
        url='https://www.8btc.com/',      # 包的主页
        description='One trading system ',   # 简单描述
        long_description=long_description,
        include_package_data=True,
        #packages=['cool_trading'],
        packages=find_packages(exclude=["test"]),                 # 包
        install_requires=[
            'redis>=2.10.5',
            'pika>=1.1.0',
        ],
        classifiers = [
            # 发展时期,常见的如下
            #   3 - Alpha
            #   4 - Beta
            #   5 - Production/Stable
            'Development Status :: 3 - Alpha',

            # 开发的目标用户
            'Intended Audience :: Developers',

            # 属于什么类型
            'Topic :: Software Development :: Build Tools',

            # 许可证信息
            'License :: OSI Approved :: MIT License',

            # 目标 Python 版本
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
        ]
)



#!/usr/bin/python

# -*- coding:utf-8 -*-

# __author__ = 'Jack'

pool = redis.ConnectionPool(host='127.0.0.1', port=6379)

r = redis.Redis(connection_pool=pool, max_connections=10)

r.flushall()  # 清空Redis

r.setex('name', value='liaogx', time=2)  # 设置新值，过期时间为3s
r.mset(k1 = 'v1', k2 = 'v2', k3 = 'v3')  # 批量设置新值
print(r.mget('k1', 'k2', 'k3', 'k4'))  # 批量获取新值
print(r.getset('name', 'liaogaoxiang'))  # 设置新值并获取原来的值
print(r.getrange('name', 0, 1))  # 获取子序列 0 <= x <= 1

r.setrange('name', 0, 'LIAO')  # 修改字符串内容，从指定字符串索引开始向后替换（新值太长时，则向后添加），返回值的长度

i = 0

while i < 4:
    print(r.get('name'))
    time.sleep(1)
    i += 1

source = 'foo'
r.set('n1', source)
r.setbit('n1', 7, 1)
'''

注：如果在Redis中有一个对应： n1 = "foo"，

    那么字符串foo的二进制表示为：01100110 01101111 01101111

    所以，如果执行 setbit('n1', 7, 1)，则就会将第7位设置为1，

    那么最终二进制则变成 01100111 01101111 01101111，即："goo"

'''
print(r.get('n1'))
print(r.getbit('n1', 7))  # 获取n1对应的值的二进制表示中的某位的值 （0或1）
r.set('n2', '廖高祥')
print(r.strlen('n2'))  # 返回对应的字节长度（一个汉字3个字节）
r.set('num', 1)
r.incr('num', amount=10)
r.decr('num', amount=1)
print(r.get('num'))  # 自增num对应的值，当name不存在时，则创建name＝amount，否则，则自增。
r.append('num', 111)
print(r.get('num'))  # 在redis num对应的值后面追加内容